'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var PropTypes = _interopDefault(require('prop-types'));

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var Shader = function Shader(_ref) {
  var antialias = _ref.antialias,
      depthTest = _ref.depthTest,
      vertex = _ref.vertex,
      fragment = _ref.fragment,
      onUpdate = _ref.onUpdate,
      onResize = _ref.onResize,
      children = _ref.children,
      style = _ref.style,
      className = _ref.className;

  var canvas = React.useRef(null);
  var wrapper = React.useRef(null);

  var _useState = React.useState(null),
      _useState2 = slicedToArray(_useState, 2),
      gl = _useState2[0],
      setGl = _useState2[1];

  var _useState3 = React.useState(null),
      _useState4 = slicedToArray(_useState3, 2),
      program = _useState4[0],
      setProgram = _useState4[1];

  var _useState5 = React.useState(0),
      _useState6 = slicedToArray(_useState5, 2),
      count = _useState6[0],
      setCount = _useState6[1];

  var _useState7 = React.useState({ width: 0, height: 0 }),
      _useState8 = slicedToArray(_useState7, 2),
      stage = _useState8[0],
      setStage = _useState8[1];

  React.useEffect(function () {
    var gl = canvas.current.getContext('webgl', { antialias: antialias });

    gl.enable(gl.BLEND);
    gl.enable(gl.CULL_FACE);
    gl.blendFunc(gl.SRC_ALPHA, gl.ONE);
    gl[depthTest ? 'enable' : 'disable'](gl.DEPTH_TEST);

    var createShader = function createShader(type, source) {
      var shader = gl.createShader(type);
      gl.shaderSource(shader, source);
      gl.compileShader(shader);
      if (gl.getShaderParameter(shader, gl.COMPILE_STATUS)) return shader;else {
        console.log(gl.getShaderInfoLog(shader));
        gl.deleteShader(shader);
      }
    };

    var program = gl.createProgram();
    gl.attachShader(program, createShader(gl.VERTEX_SHADER, vertex));
    gl.attachShader(program, createShader(gl.FRAGMENT_SHADER, fragment));
    gl.linkProgram(program);

    if (gl.getProgramParameter(program, gl.LINK_STATUS)) gl.useProgram(program);else {
      console.log(gl.getProgramInfoLog(program));
      gl.deleteProgram(program);
    }

    setGl(gl);
    setProgram(program);
  }, []);

  React.useEffect(function () {
    var startTime = performance.now();
    var oldTime = performance.now();
    var raf = null;

    var frame = function frame() {
      raf = requestAnimationFrame(frame);

      var now = performance.now();
      var delta = now - oldTime;
      var elapsed = (now - startTime) / 5000;
      oldTime = now;

      onUpdate(delta, elapsed);
      if (count) gl.drawArrays(gl.POINTS, 0, count);
    };

    raf = requestAnimationFrame(frame);
    return function () {
      return cancelAnimationFrame(raf);
    };
  }, [gl, program, count]);

  React.useEffect(function () {
    if (!gl || !program) return;

    var onShaderResize = function onShaderResize() {
      var dpi = window.devicePixelRatio;
      var _wrapper$current = wrapper.current,
          offsetWidth = _wrapper$current.offsetWidth,
          offsetHeight = _wrapper$current.offsetHeight;

      var width = offsetWidth * dpi;
      var height = offsetHeight * dpi;

      canvas.current.width = width;
      canvas.current.height = height;

      gl.viewport(0, 0, width, height);
      gl.clearColor(0, 0, 0, 0);

      onResize(offsetWidth, offsetHeight, dpi);
      setStage({ width: width, height: height });
    };
    onShaderResize();

    window.addEventListener('resize', onShaderResize);
    return function () {
      return window.removeEventListener('resize', onShaderResize);
    };
  }, [gl, program]);

  return React__default.createElement(
    'div',
    { ref: wrapper, style: _extends({}, style), className: className },
    React__default.createElement('canvas', { ref: canvas, style: { display: 'block', width: '100%', height: '100%' } }),
    gl !== null && program !== null && React__default.Children.map(children, function (child) {
      return React__default.cloneElement(child, { gl: gl, program: program, stage: stage, setCount: setCount });
    })
  );
};

Shader.propTypes = {
  antialias: PropTypes.bool,
  depthTest: PropTypes.bool,
  vertex: PropTypes.string.isRequired,
  fragment: PropTypes.string.isRequired,
  onUpdate: PropTypes.func,
  onResize: PropTypes.func,
  style: PropTypes.object,
  className: PropTypes.string,
  children: PropTypes.node
};

Shader.defaultProps = {
  antialias: true,
  depthTest: false,
  vertex: '',
  fragment: '',
  onUpdate: function onUpdate(delta, elapsed) {},
  onResize: function onResize(width, height, dpi) {},
  style: {},
  className: ''
};

var Uniform = function Uniform(_ref) {
  var name = _ref.name,
      type = _ref.type,
      value = _ref.value,
      gl = _ref.gl,
      program = _ref.program;

  var data = React.useRef({ update: function update() {} });

  React.useEffect(function () {
    var location = gl.getUniformLocation(program, name);
    var update = {
      'int': function int(value) {
        return gl.uniform1i(location, value);
      },
      'float': function float(value) {
        return gl.uniform1f(location, value);
      },
      'vec2': function vec2(value) {
        return gl.uniform2f.apply(gl, [location].concat(toConsumableArray(value)));
      },
      'vec3': function vec3(value) {
        return gl.uniform3f.apply(gl, [location].concat(toConsumableArray(value)));
      },
      'vec4': function vec4(value) {
        return gl.uniform4f.apply(gl, [location].concat(toConsumableArray(value)));
      },
      'mat2': function mat2(value) {
        return gl.uniformMatrix2fv(location, false, value);
      },
      'mat3': function mat3(value) {
        return gl.uniformMatrix3fv(location, false, value);
      },
      'mat4': function mat4(value) {
        return gl.uniformMatrix4fv(location, false, value);
      }
    }[type];

    data.current = { update: update };
  }, []);

  React.useEffect(function () {
    var update = data.current.update;

    update(value);
  }, [value]);

  return null;
};

Uniform.propTypes = {
  name: PropTypes.string.isRequired,
  type: PropTypes.oneOf(['int', 'float', 'vec2', 'vec3', 'vec4', 'mat2', 'mat3', 'mat4']).isRequired,
  value: PropTypes.oneOfType([PropTypes.number, PropTypes.array]).isRequired
};

var Attribute = function Attribute(_ref) {
  var _ref$name = _ref.name,
      name = _ref$name === undefined ? '' : _ref$name,
      _ref$size = _ref.size,
      size = _ref$size === undefined ? 3 : _ref$size,
      _ref$value = _ref.value,
      value = _ref$value === undefined ? [] : _ref$value,
      _ref$main = _ref.main,
      main = _ref$main === undefined ? false : _ref$main,
      gl = _ref.gl,
      program = _ref.program,
      setCount = _ref.setCount;

  var data = React.useRef({ update: function update() {} });

  React.useEffect(function () {
    var index = gl.getAttribLocation(program, name);
    var buffer = gl.createBuffer();

    gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
    gl.enableVertexAttribArray(index);
    gl.vertexAttribPointer(index, size, gl.FLOAT, false, 0, 0);

    var update = function update(value) {
      gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(value), gl.STATIC_DRAW);
    };

    data.current = { update: update };
  }, []);

  React.useEffect(function () {
    var update = data.current.update;

    if (main === true) setCount(value.length / size);
    update(value);
  }, [value]);

  return null;
};

Attribute.propTypes = {
  name: PropTypes.string.isRequired,
  size: PropTypes.number.isRequired,
  value: PropTypes.array.isRequired
};

var Texture = function Texture(_ref) {
  var _ref$src = _ref.src,
      src = _ref$src === undefined ? null : _ref$src,
      gl = _ref.gl;

  React.useEffect(function () {
    var texture = gl.createTexture();
    gl.bindTexture(gl.TEXTURE_2D, texture);
    gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, 1, 1, 0, gl.RGBA, gl.UNSIGNED_BYTE, new Uint8Array([0, 0, 0, 0]));

    var img = new Image();
    img.onload = function () {
      gl.bindTexture(gl.TEXTURE_2D, texture);
      gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, img);
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
      gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
    };
    img.src = src;
  }, []);

  return null;
};

Texture.propTypes = {
  src: PropTypes.string.isRequired
};

var Camera = function Camera(_ref) {
  var fov = _ref.fov,
      near = _ref.near,
      far = _ref.far,
      position = _ref.position,
      target = _ref.target,
      perspective = _ref.perspective,
      gl = _ref.gl,
      program = _ref.program,
      _ref$stage = _ref.stage,
      width = _ref$stage.width,
      height = _ref$stage.height;

  var _useState = React.useState(Array.from({ length: 16 })),
      _useState2 = slicedToArray(_useState, 2),
      projection = _useState2[0],
      setProjection = _useState2[1];

  React.useEffect(function () {
    setProjection(createProjection({
      width: width,
      height: height,
      perspective: perspective,
      fov: fov,
      near: near,
      far: far,
      position: position,
      target: target
    }));
  }, [fov, near, far, position, perspective, width, height]);

  return React__default.createElement(Uniform, { name: 'u_projection', type: 'mat4', value: projection, gl: gl, program: program });
};

Camera.propTypes = {
  fov: PropTypes.number,
  near: PropTypes.number,
  far: PropTypes.number,
  position: PropTypes.array,
  target: PropTypes.array,
  perspective: PropTypes.bool,
  gl: PropTypes.any,
  program: PropTypes.any,
  stage: PropTypes.object
};

Camera.defaultProps = {
  fov: 60,
  near: 1,
  far: 10000,
  position: [0, 0, 100],
  target: [0, 0, 0],
  perspective: true
};

var createProjection = function createProjection(_ref2) {
  var width = _ref2.width,
      height = _ref2.height,
      _ref2$fov = _ref2.fov,
      fov = _ref2$fov === undefined ? 60 : _ref2$fov,
      _ref2$near = _ref2.near,
      near = _ref2$near === undefined ? 1 : _ref2$near,
      _ref2$far = _ref2.far,
      far = _ref2$far === undefined ? 10000 : _ref2$far,
      _ref2$position = _ref2.position,
      position = _ref2$position === undefined ? [0, 0, 100] : _ref2$position,
      _ref2$target = _ref2.target,
      target = _ref2$target === undefined ? [0, 0, 0] : _ref2$target,
      _ref2$perspective = _ref2.perspective,
      perspective = _ref2$perspective === undefined ? true : _ref2$perspective;

  if (perspective) {
    var aspect = width / height;

    var fovRad = fov * (Math.PI / 180);
    var f = Math.tan(Math.PI / 2 - fovRad / 2);
    var rangeInv = 1.0 / (near - far);

    var projectionMatrix = [f / aspect, 0, 0, 0, 0, f, 0, 0, 0, 0, (near + far) * rangeInv, -1, 0, 0, near * far * rangeInv * 2, 0];

    var normalize = function normalize(v) {
      var length = Math.sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
      return length > 0 ? [v[0] / length, v[1] / length, v[2] / length] : [0, 0, 0];
    };

    var cross = function cross(a, b) {
      return [a[1] * b[2] - a[2] * b[1], a[2] * b[0] - a[0] * b[2], a[0] * b[1] - a[1] * b[0]];
    };

    var subtract = function subtract(a, b) {
      return [a[0] - b[0], a[1] - b[1], a[2] - b[2]];
    };

    var multiply = function multiply(a, b) {
      return a.reduce(function (acc, val, idx) {
        var i = Math.floor(idx / 4) * 4;
        var j = idx % 4;
        acc[idx] = b[i] * a[j] + b[i + 1] * a[j + 4] + b[i + 2] * a[j + 8] + b[i + 3] * a[j + 12];
        return acc;
      }, []);
    };

    var inverse = function inverse(a) {
      var b = [a[0] * a[5] - a[1] * a[4], a[0] * a[6] - a[2] * a[4], a[0] * a[7] - a[3] * a[4], a[1] * a[6] - a[2] * a[5], a[1] * a[7] - a[3] * a[5], a[2] * a[7] - a[3] * a[6], a[8] * a[13] - a[9] * a[12], a[8] * a[14] - a[10] * a[12], a[8] * a[15] - a[11] * a[12], a[9] * a[14] - a[10] * a[13], a[9] * a[15] - a[11] * a[13], a[10] * a[15] - a[11] * a[14]];

      var det = 1.0 / (b[0] * b[11] - b[1] * b[10] + b[2] * b[9] + b[3] * b[8] - b[4] * b[7] + b[5] * b[6]);

      return [(a[5] * b[11] - a[6] * b[10] + a[7] * b[9]) * det, (a[2] * b[10] - a[1] * b[11] - a[3] * b[9]) * det, (a[13] * b[5] - a[14] * b[4] + a[15] * b[3]) * det, (a[10] * b[4] - a[9] * b[5] - a[11] * b[3]) * det, (a[6] * b[8] - a[4] * b[11] - a[7] * b[7]) * det, (a[0] * b[11] - a[2] * b[8] + a[3] * b[7]) * det, (a[14] * b[2] - a[12] * b[5] - a[15] * b[1]) * det, (a[8] * b[5] - a[10] * b[2] + a[11] * b[1]) * det, (a[4] * b[10] - a[5] * b[8] + a[7] * b[6]) * det, (a[1] * b[8] - a[0] * b[10] - a[3] * b[6]) * det, (a[12] * b[4] - a[13] * b[2] + a[15] * b[0]) * det, (a[9] * b[2] - a[8] * b[4] - a[11] * b[0]) * det, (a[5] * b[7] - a[4] * b[9] - a[6] * b[6]) * det, (a[0] * b[9] - a[1] * b[7] + a[2] * b[6]) * det, (a[13] * b[1] - a[12] * b[3] - a[14] * b[0]) * det, (a[8] * b[3] - a[9] * b[1] + a[10] * b[0]) * det];
    };

    var z = normalize(subtract(position, target));
    var x = normalize(cross([0, 1, 0], z));
    var y = normalize(cross(z, x));

    var cameraMatrix = [x[0], x[1], x[2], 0, y[0], y[1], y[2], 0, z[0], z[1], z[2], 0, position[0], position[1], position[2], 1];

    return multiply(projectionMatrix, inverse(cameraMatrix));
  } else {
    return [2 / width, 0, 0, 0, 0, -2 / height, 0, 0, 0, 0, 1, 0, -1, 1, 0, 1];
  }
};

exports.Shader = Shader;
exports.Uniform = Uniform;
exports.Attribute = Attribute;
exports.Texture = Texture;
exports.Camera = Camera;
exports.createProjection = createProjection;
//# sourceMappingURL=index.js.map
